## Puppet-Riak Development Roadmap

The purpose of this roadmap is to document long-term vision and goals for this module. This will attempt to be in sync with the [basho-labs config management strategy](https://github.com/basho-labs/the-riak-community/blob/master/config-mgmt-strategy.md).


What would a custom type provide information on?

riak config effective # capture current configuration status
show cluster members


functionality
- join cluster
- leave cluster
- create backups in a cron job
- security
  - generally enable / disable (and see current status)
  - enable SSL
    - use existing Puppet SSL infrastructure for SSL
    - should be able to get CRL
  - security-sources
    - trust-based auth
    - manage users for password-based auth
    - manage pam-based auth
  - ability to purge users, groups, grants, etc not managed by puppet

examples that should be documented:
- how to use puppetlabs-haproxy to puppetize a load balancer for riak
- how to use puppetlabs-ntp to get node time in sync

resource types:
riak_user { 'example':
  username => 'daniel', # defaults to resource name, this is just to show it can be different
  groups   => ['admins', 'developers'],
  password => 'hunter2',
  options  => {
    'name' => 'lucius',
    'fav_color' => 'red',
  },
  permissions => {
    bucket_type   => '*',
    bucket => '*',
    grants => [ 'riak_kv.get', 'riak_kv.delete', 'riak_kv.put']
  },
}

riak_group { 'admins':
  name => 'admins',
  permissions => {
    type   => '*',
    bucket => '*',
    grants => [ 'riak_kv.get', 'riak_kv.delete', 'riak_kv.put']
  },
  groups => 'everybody',
}

riak_security_source { 'localhost':
  users   => all,
  type    => 'trust',
  cidr    => '127.0.0.1/32',
  options => []
}

riak_security_source { 'private network':
  users   => all,
  type    => 'password',
  cidr    => '10.0.0.0/24',
  options => []
}

riak_security_source { 'WAN':
  users   => ['riakuser', 'daniel']
  type    => 'certificate',
  cidr    => '10.1.0.0/24',
  options => []
}

riak_security {
  ciphers => 'DHE-RSA-AES256-SHA:AES128-GCM-SHA256',
  # any other settings?
}

riak_bucket_type {'n_equals_1':
  ensure => 'activated',
  props  => {
    'n_val'      => '1',
    'allow_mult' => true,
    'precommit'  => ["syntax_check"],
    'postcommit' => ["welcome_email", "update_registry"]
  },
  max_propagation_wait => 30,
}
